@@ -192,7 +192,7 @@ void exit_sliplogin()
 
 int main(int argc, char **argv)
 {
-    int fd, s, ldisc, odisc, pid, unitnumber;
+    int fd, s, ldisc, odisc, pid, unitnumber, i;
     speed_t speed;
     struct sigaction sa;
     char *name;
@@ -212,11 +212,10 @@ int main(int argc, char **argv)
 #endif
     extern uid_t getuid();
 
-    if ((name = strrchr(argv[0], '/')) == NULL)
-      name = argv[0];
+    name = strrchr(argv[0], '/');
+    if (name == NULL) name = argv[0]; else name++;
     s = (int) getdtablesize();
-    for (fd = 3 ; fd < s ; fd++)
-      (void) close(fd);
+    for (fd = 3 ; fd < s ; fd++) (void) close(fd);
     openlog(name, LOG_PID, LOG_DAEMON);
     info.uid = getuid();
 
@@ -225,81 +224,82 @@ int main(int argc, char **argv)
 
     if (argc > 1) 
     {
-	if (info.uid!=0) 
-	{
-	    fprintf(stderr,"Only root may use arguments\n");
-	    exit(-1);
-	}
-	info.loginname=argv[1];
-	scan_sliphosts(&info);
+	 if (info.uid!=0) 
+	 {
+	     fprintf(stderr,"Only root may use arguments\n");
+	     exit(-1);
+	 }
+	 info.loginname=argv[1];
+	 scan_sliphosts(&info);
 
 	/*
 	 * Disassociate from current controlling terminal, if any,
 	 * and ensure that the slip line is our controlling terminal.
 	 */
 #ifdef POSIX
-	if (fork() > 0)
-	  exit(0);
+	if (fork() > 0) exit(0);
 
 	if (setsid() < 0) syslog(LOG_ERR,"can't setsid: %m");
 #else
-	if ((fd = open("/dev/tty", O_RDONLY, 0)) >= 0) {
+	if ((fd = open("/dev/tty", O_RDONLY, 0)) >= 0)
+    {
 	    extern char *ttyname();
 
 	    (void) ioctl(fd, TIOCNOTTY, (caddr_t)0);
 	    (void) close(fd);
 	    /* open slip tty again to acquire as controlling tty? */
 	    fd = open(ttyname(0), O_RDWR, 0);
-	    if (fd >= 0)
-	      (void) close(fd);
+	    if (fd >= 0) (void) close(fd);
 	}
 	(void) setpgrp(0, getpid());
 #endif
 	/* open the line as stdin, if given as argument */
-	if (argc > 2) { 
-	    if ((fd = open(argv[2], O_RDWR)) == -1) {
-		perror(argv[2]);
-		exit(2);
+	if (argc > 2)
+    { 
+	    if ((fd = open(argv[2], O_RDWR)) == -1)
+        {
+		 perror(argv[2]);
+		 exit(2);
 	    }
 	    (void) dup2(fd, 0);
-	    if (fd > 2)
-	      close(fd);
+	    if (fd > 2) close(fd);
 	}
 #ifdef TIOCSCTTY
 	if (ioctl(0, TIOCSCTTY, (caddr_t)0) < 0)
 	  perror("sliplogin: can't set my controlling tty -");
 #endif
-    }
+    } /* endif argc > 1 */
     else
     {
-	extern char *getlogin();
 	/* Warning: Extended logins '[oca]slip' will only work
-	 * with different uid's when using getlogin() here!
+	 * with different uid's.
 	 */
-	if ((name = getlogin()) == NULL) {
+     pw = getpwuid(info.uid);
+	 if (pw == NULL)
+     {
 	    (void) fprintf(stderr, "access denied - no username\n");
 	    syslog(LOG_ERR, "access denied - getlogin returned 0\n");
 	    FATAL(1);
-	}
+	 }
+	 info.loginname = pw->pw_name;
+     i=strlen(info.loginname);
 #ifdef ESLIPLOGIN
-	/* anonymous login, get host(user)-name and password */
-	if (strstr(name, "slip"))
-	{	    
+	 /* anonymous login, get host(user)-name and password */
+	 if ( (i>4) && (strstr(info.loginname+i-5, "SLIP")!=NULL) )
+	 {	    
 	    void esliplogin (struct slipinfo *sinfo, char* name);
 	    esliplogin (&info, name);
-	} else
+	 } else
 #endif
-	{
-	    info.loginname = name;
-	    scan_sliphosts(&info);
-	}
-    }
+	 scan_sliphosts(&info);
+    } /* end else */
 
     /* Old BSD systems cannot do SLIP over a pseudo tty ... */
-    if (!isatty(0)) {
-	(void) fprintf(stderr, "access denied - stdin is not a tty\n");
-	syslog(LOG_ERR, "access denied - stdin is not a tty\n");
-	FATAL(1);
+    if (!isatty(0))
+    {
+	 (void) fprintf(stderr, "access denied - stdin is not a tty\n");
+	 syslog(LOG_ERR, "access denied - stdin is not a tty\n");
+	 FATAL(1);
     }
 
 #ifdef STATS    
@@ -341,62 +341,66 @@ int main(int argc, char **argv)
 
 #ifdef POSIX
     /* set up the line parameters */
-    if (tcgetattr(0, &tios) < 0) {
-	syslog(LOG_ERR, "tcgetattr: %m");
-	exit(1);
+    if (tcgetattr(0, &tios) < 0)
+    {
+	 syslog(LOG_ERR, "tcgetattr: %m");
+	 exit(1);
     }
     otios = tios;
     cfmakeraw(&tios);
     tios.c_iflag &= ~IMAXBEL;
-    if (tcsetattr(0, TCSAFLUSH, &tios) < 0) {
-	syslog(LOG_ERR, "tcsetattr: %m");
-	exit(1);
+    if (tcsetattr(0, TCSAFLUSH, &tios) < 0)
+    {
+	 syslog(LOG_ERR, "tcsetattr: %m");
+	 exit(1);
     }
     speed = cfgetispeed(&tios);
     info.linespeed=translate_speed(speed);
 #else
     /* set up the line parameters */
-    if (ioctl(0, TIOCGETP, (caddr_t)&tty) < 0) {
-	syslog(LOG_ERR, "ioctl (TIOCGETP): %m");
-	exit(1);
+    if (ioctl(0, TIOCGETP, (caddr_t)&tty) < 0)
+    {
+	 syslog(LOG_ERR, "ioctl (TIOCGETP): %m");
+	 exit(1);
     }
     otty = tty;
     speed = tty.sg_ispeed;
     info.linespeed=translate_speed(speed);
     tty.sg_flags = RAW | ANYP;
-    if (ioctl(0, TIOCSETP, (caddr_t)&tty) < 0) {
-	syslog(LOG_ERR, "ioctl (TIOCSETP): %m");
-	exit(1);
+    if (ioctl(0, TIOCSETP, (caddr_t)&tty) < 0)
+    {
+	 syslog(LOG_ERR, "ioctl (TIOCSETP): %m");
+	 exit(1);
     }
 #endif
 
     /* find out what ldisc we started with */
     if (ioctl(0, TIOCGETD, (caddr_t)&odisc) < 0)
     {
-	syslog(LOG_ERR, "ioctl(TIOCGETD) (1): %m");
-	exit(1);
+	 syslog(LOG_ERR, "ioctl(TIOCGETD) (1): %m");
+	 exit(1);
     }
     /* switch to slip line discipline */
     ldisc = N_SLIP;
     if (ioctl(0, TIOCSETD, (caddr_t)&ldisc) < 0)
     {
-	syslog(LOG_ERR, "ioctl(TIOCSETD): %m");
-	exit(1);
+	 syslog(LOG_ERR, "ioctl(TIOCSETD): %m");
+	 exit(1);
     }
     
     /* find out what unit number we were assigned */
 #if defined(SLIOCGUNIT)
     if (ioctl(0, SLIOCGUNIT, (caddr_t) &unitnumber) < 0)
     {
-	syslog(LOG_ERR, "ioctl (SLIOCGUNIT): %m");
-	exit(1);
+	 syslog(LOG_ERR, "ioctl (SLIOCGUNIT): %m");
+	 exit(1);
     }
     sprintf(info.unit,"sl%d",unitnumber);
 #elif defined(SIOCGIFNAME)
     if (ioctl(0, SIOCGIFNAME, (caddr_t) info.unit) < 0)
     {
-	syslog(LOG_ERR, "ioctl (SIOCGIFNAME): %m");
-	exit(1);
+	 syslog(LOG_ERR, "ioctl (SIOCGIFNAME): %m");
+	 exit(1);
     }
 #endif
     
@@ -404,8 +408,8 @@ int main(int argc, char **argv)
     sa.sa_flags = 0;
     if (sigemptyset(&sa.sa_mask) != 0)
     {
-	syslog(LOG_ERR, "I have no signal control: %m");
-	exit(1);
+	 syslog(LOG_ERR, "I have no signal control: %m");
+	 exit(1);
     }
 
     sigaction(SIGHUP,&sa,NULL);
@@ -423,13 +427,15 @@ int main(int argc, char **argv)
      * babble into the slip tty line.
      */
     close(1);
-    if ((fd = open(_PATH_DEVNULL, O_WRONLY)) != 1) {
-	if (fd < 0) {
+    if ((fd = open(_PATH_DEVNULL, O_WRONLY)) != 1)
+    {
+	 if (fd < 0)
+     {
 	    syslog(LOG_ERR, "open /dev/null: %m");
 	    exit(1);
-	}
-	(void) dup2(fd, 1);
-	(void) close(fd);
+	 }
+	 (void) dup2(fd, 1);
+	 (void) close(fd);
     }
     (void) dup2(1, 2);
 
@@ -442,19 +448,25 @@ int main(int argc, char **argv)
     s=system(logincmd);
     if (s!=0)
     {
-	syslog(LOG_ERR, "%s login failed: exit status %d from %s",
+	 syslog(LOG_ERR, "%s login failed: exit status %d from %s",
 	       info.loginname, s, info.loginfile);
-	ioctl(0, TIOCSETD, (caddr_t)&odisc);
-	exit(6);
+	 ioctl(0, TIOCSETD, (caddr_t)&odisc);
+	 exit(6);
     }
     
     /* set slip mode (FreeBSD 2.x: ifconfig flag normal/compressed/link2) */
 #if defined(SLIOCSFLAGS)
-    if (ioctl(0, SLIOCSFLAGS, (caddr_t)&info.sm_value) < 0) {
-	syslog(LOG_ERR, "ioctl (SLIOCSFLAGS): %m"); exit(1); }
+    if (ioctl(0, SLIOCSFLAGS, (caddr_t)&info.sm_value) < 0)
+    {
+	 syslog(LOG_ERR, "ioctl (SLIOCSFLAGS): %m");
+     exit(1);
+    }
 #elif defined(SIOCSIFENCAP)
-    if (ioctl(0, SIOCSIFENCAP, (caddr_t)&info.sm_value) < 0) {
-	syslog(LOG_ERR, "ioctl (SIOCSIFENCAP): %m"); exit(1); }
+    if (ioctl(0, SIOCSIFENCAP, (caddr_t)&info.sm_value) < 0)
+    {
+	 syslog(LOG_ERR, "ioctl (SIOCSIFENCAP): %m");
+     exit(1);
+    }
 #endif
 
     /* reset uid to users' to allow the user to give a signal.
